 import java.util.Enumeration;
import java.util.List;

import weka.associations.Apriori;
import weka.associations.AssociationRule;
import weka.associations.FPGrowth;
import weka.classifiers.evaluation.Evaluation;
import weka.classifiers.trees.J48;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.SelectedTag;
import weka.core.converters.ConverterUtils.DataSink;
import weka.core.converters.ConverterUtils.DataSource;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;


public class WekaAPISample {

	public static void main(String[] args) throws Exception {
		
		//Loading of data from a file requires to import 
		// weka.core.converters.ConverterUtils.DataSource;
		Instances dataset = DataSource.read("census.arff");
		int natt = dataset.numAttributes();
		System.out.println("dataset has " + natt + " attributes");
		System.out.println("dataset has " + dataset.numInstances() + " rows");
		
		//for taking a row, the first one
		Instance r1 = dataset.get(0);
		//write in csv format the first row
		for(int i=0; i<natt-1; ++i)
			System.out.print(r1.toString(i)+",");
		System.out.println(r1.toString(natt-1));
		
		//take the last attribute by postition: it is an enumerated attribute
		Attribute last = dataset.attribute(natt-1);
		//take from the attribute the enumerated values and put in an object that may be
		//scan by a while
		Enumeration<Object> v = last.enumerateValues();
		while(v.hasMoreElements()) 
			System.out.println(v.nextElement().toString());

		//FILTERS
		//supervised filter
		Remove r = new Remove();
		r.setOptions("-R 1".split(" "));
		//it is necessary to pass the format of data
		r.setInputFormat(dataset);
		//for applying the filter useFilter and returns another dataset with the
		//filter applyied
		Instances removeddataset = Filter.useFilter(dataset, r);
		System.out.println("removeddataset has " + removeddataset.numAttributes() + " attributes");
		System.out.println("removeddataset has " + removeddataset.numInstances() + " rows");
		
		//save data in a file
		DataSink.write("censusremoved.arff", removeddataset);
		
		//Classification
		dataset.setClassIndex(natt-1); //set the class for classification
		J48 cls = new J48();
		cls.setMinNumObj(200);
		cls.buildClassifier(dataset); //take the training set
		
		System.out.println(cls.toString()); //representation of the tree
		
		//EVALUATION
		Evaluation eval = new Evaluation(dataset);
		eval.evaluateModel(cls, dataset
		
		System.out.println(eval.toSummaryString());		
		
		//System.out.println(eval.toSummaryString("\nResults\n\n", false));		
		
		//we ask to the dataset the class attribute
		Attribute classatt = dataset.classAttribute();
		//Prediction of an instance: returns the index of the enumerated 
		//value. In case of numerical prediction the returned values is the 
		//final prediction and not the index.
		double cp = cls.classifyInstance(r1);
		//to print the class use classatt.value method
		System.out.println("Class predicted = " + classatt.value((int)cp));
		
		//the prediction is a distribution of probability to be assigned to a class
		double[] d = cls.distributionForInstance(r1);
		for(int i=0; i<d.length; ++i)
			System.out.println("Confidence of prediction " + classatt.value(i) + " = " + d[i]);
		
		Instances baskets = DataSource.read("supermarket.arff");
		FPGrowth ar = new FPGrowth();
		ar.setNumRulesToFind(10);
		ar.setUpperBoundMinSupport(1);
		ar.setLowerBoundMinSupport(0.05);
		ar.setDelta(0.01);
		ar.setMinMetric(0.8);
		ar.setMetricType(new SelectedTag(0, Apriori.TAGS_SELECTION)); // 0 = CONFIDENCE, 1 = LIFT
		ar.buildAssociations(baskets);
		System.out.println(ar);

		List<AssociationRule> rules = ar.getAssociationRules().getRules();
		for(AssociationRule rule : rules)
			System.out.println(rule);
		
	}

}
